@@ -52,9 +52,9 @@ class PostFormula(object):
   def checklimited(self, contents, index):
     "Check for a command with limits"
     bit = contents[index]
-    if not hasattr(bit, 'command'):
+    if not isinstance(bit, EmptyCommand):
       return
-    if not bit.command in FormulaConfig.limited:
+    if not bit.command in FormulaConfig.limits['commands']:
       return
     limits = self.findlimits(contents, index + 1)
     limits.reverse()
@@ -76,9 +76,9 @@ class PostFormula(object):
   def checklimits(self, contents, index):
     "Check for a command making the limits"
     bit = contents[index]
-    if not hasattr(bit, 'command'):
+    if not isinstance(bit, SymbolFunction):
       return False
-    if not bit.command in FormulaConfig.limits:
+    if not bit.command in FormulaConfig.limits['operands']:
       return False
     bit.output.tag += ' class="bigsymbol"'
     return True
@@ -86,9 +86,9 @@ class PostFormula(object):
   def checkroot(self, contents, index):
     "Check for a root, insert the radical in front"
     bit = contents[index]
-    if not hasattr(bit, 'original'):
+    if not hasattr(bit, 'mark'):
       return
-    if not bit.original.startswith(FormulaConfig.starts['root']):
+    if bit.mark != 'sqrt':
       return
     radical = TaggedText().constant(u'√', 'span class="radical"')
     root = TaggedText().complete(bit.contents, 'span class="root"')
@@ -99,11 +99,11 @@ class PostFormula(object):
     label = contents[index]
     if not isinstance(label, LabelFunction):
       return
-    if len(label.contents) != 1:
+    if len(label.contents) < 1 or not isinstance(label.contents[0], Bracket):
       Trace.error('Wrong contents for label ' + unicode(label))
       return
     bracket = label.contents[0]
-    labelname = bracket.contents
+    labelname = bracket.literal
     number = '(' + self.generator.generate(1) + ') '
     Label.names[labelname] = label
     tag = label.output.tag.replace('#', labelname)
