@@ -77,7 +77,7 @@ class FormulaParser(Parser):
         return self.parsemultiliner(reader, startpiece, endpiece)
       Trace.error('Missing ' + beginafter + ' in ' + reader.currentline())
       return ''
-    begincommand = FormulaConfig.starts['FormulaCommand']
+    begincommand = FormulaConfig.starts['command']
     beginbracket = FormulaConfig.starts['bracket']
     if begincommand in reader.currentline() and beginbracket in reader.currentline():
       endbracket = FormulaConfig.endings['bracket']
@@ -122,6 +122,7 @@ class Position(object):
   def __init__(self, text):
     self.text = text
     self.pos = 0
+    self.endinglist = EndingList()
 
   def skip(self, string):
     "Skip a string"
@@ -131,12 +132,22 @@ class Position(object):
     "Return the text remaining for parsing"
     return self.text[self.pos:]
 
+  def finished(self):
+    "Find out if the current formula has finished"
+    if self.isout():
+      self.endinglist.checkpending()
+      return True
+    return self.endinglist.checkin(self)
+
   def isout(self):
     "Find out if we are out of the formula yet"
     return self.pos >= len(self.text)
 
   def current(self):
     "Return the current character"
+    if self.isout():
+      Trace.error('Out of the formula')
+      return ''
     return self.text[self.pos]
 
   def checkfor(self, string):
@@ -145,10 +156,80 @@ class Position(object):
       return False
     return self.text[self.pos : self.pos + len(string)] == string
 
-  def clone(self):
-    "Return a copy of self"
-    clone = Position(self.text)
-    clone.pos = self.pos
-    return clone
+  def pushending(self, ending, optional = False):
+    "Push a new ending to the bottom"
+    self.endinglist.add(ending, optional)
 
+  def popending(self, expected = None):
+    "Pop the ending found at the current position"
+    ending = self.endinglist.pop(self)
+    if expected and expected != ending:
+      Trace.error('Expected ending ' + expected + ', got ' + ending)
+    self.skip(ending)
+    return ending
+
+class EndingList(object):
+  "A list of position endings"
+
+  def __init__(self):
+    self.endings = []
+
+  def add(self, ending, optional):
+    "Add a new ending to the list"
+    self.endings.append(PositionEnding(ending, optional))
+
+  def checkin(self, pos):
+    "Search for an ending"
+    if self.findending(pos):
+      return True
+    return False
+
+  def pop(self, pos):
+    "Remove the ending at the current position"
+    ending = self.findending(pos)
+    if not ending:
+      Trace.error('No ending at ' + pos.remaining())
+      return ''
+    for each in reversed(self.endings):
+      self.endings.remove(each)
+      if each == ending:
+        return each.ending
+      elif not each.optional:
+        Trace.error('Removed non-optional ending ' + each)
+    Trace.error('No endings left')
+    return ''
+
+  def findending(self, pos):
+    "Find the ending at the current position"
+    if len(self.endings) == 0:
+      return None
+    for index, ending in enumerate(reversed(self.endings)):
+      if ending.checkin(pos):
+        return ending
+      if not ending.optional:
+        return None
+    return None
+
+  def checkpending(self):
+    "Check if there are any pending endings"
+    if len(self.endings) != 0:
+      Trace.error('Pending endings ' + unicode(self.endings) + ' in ' + self.text)
+
+class PositionEnding(object):
+  "An ending for a formula position"
+
+  def __init__(self, ending, optional):
+    self.ending = ending
+    self.optional = optional
+
+  def checkin(self, pos):
+    "Check for the ending"
+    return pos.checkfor(self.ending)
+
+  def __unicode__(self):
+    "Printable representation"
+    string = 'Ending ' + self.ending
+    if self.optional:
+      string += ' (optional)'
+    return string
 
